"""
Streamlit Application
Marie FU
"""

import streamlit as st
import csv
import codecs
import src.basic as tmp
import json
from src.gerneralFileOP import *
from src.csvFileOP import *

# Define constant for file encoding supported (name in lower case)
FILE_ENCODING = ["utf-8", "utf-8-sig", "ascii"]

CSV_COLUMNS = ["status", "label", "forms"]

STATE = st.session_state

tmp.base("CleanCSVTerms")


def lowerName(name : str) -> str:

    """
    Set a given String to lowercase and removes unecessary blank spaces

    Args:
        name (String) : The given string to format

    Returns:
        res (String) : The changed string

    Authors:
        Marie FU
    """
    
    return " ".join(name.split()).lower()


def getContent() -> None:

    """
    Get the columns present in the given file

    Args:
        separator (String) : Used separator of the file

    Returns:
        None

    Authors:
        Marie FU
    """
    
    csvLine = 2

    reader = csv.DictReader(codecs.iterdecode(STATE.file, STATE.encoding), delimiter=STATE.separator)
    for row in reader:
        to_add = ""
        for name, value in row.items():
            if name in STATE.columnMap.keys() and value is not None:
                if STATE.columnMap[name] == "label" and "\"" in value:
                    to_add += "\"" + value.replace('"','""') + "\"\t"
                else:
                    to_add += f"{value}\t"
        to_add = to_add[:-1] + "\n"
        if to_add not in STATE.fileData:
            STATE.fileData += to_add
        csvLine += 1
    return


st.write(st.session_state.general_text_dict['description'])

STATE.newFileName = st.text_input(label=st.session_state.general_text_dict['file_name_input'],placeholder="File.json")

st.file_uploader(st.session_state.general_text_dict['file'], type=["tsv", "csv"], key="file")

# checking if a file is uploaded
if STATE.file is not None:

    # make sure to be at start of file before any operation
    # file pointer will be reset each time after each file operation
    toStartOfFile(STATE.file)

    # first utilisation of errMessageLog, contain the log of every error message encountered
    STATE.errMessageLog = ""

    # check new file name given by user
    STATE.newFileName = checkNewFileName(STATE.newFileName, STATE.file.name)
    if STATE.newFileName is None:
        STATE.errMessageLog += st.session_state.general_text_dict['err_file_name']

    # check file encoding
    STATE.encoding = checkEncoding(STATE.file)
    toStartOfFile(STATE.file)
    if STATE.encoding is None:
        STATE.errMessageLog += st.session_state.general_text_dict['err_file_encoding']

    # continue only if file name and file encoding are in a correct
    if STATE.errMessageLog != "":
        errDisplay(STATE.errMessageLog)
    elif STATE.encoding is not None and STATE.newFileName is not None:
        # get csv separator
        STATE.separator = getSeparator(STATE.file, STATE.encoding)
        toStartOfFile(STATE.file)
        # continue only if separator was found and correct
        if STATE.separator is None:
            STATE.errMessageLog += STATE.general_text_dict['err_column_separator']
            errDisplay(STATE.errMessageLog)
        else :
            # get the mapping of the files columns name to the right columns name
            STATE.columnMap = {}
            STATE.columnMap = getColumnsNames(STATE.file, STATE.encoding, STATE.separator, CSV_COLUMNS)
            if STATE.columnMap != {} and STATE.columnMap != {'choisir une option': 'Abstract'}:

                toStartOfFile(STATE.file)

                STATE.fileData = ""
                addColumnsNamestoTSV(STATE.file, STATE.encoding, STATE.separator,STATE.columnMap)

                toStartOfFile(STATE.file)

                # set a variable to check if there are any detected errors
                # and construction of correct file content while checking
                STATE.contentProblem = False
                getContent()

                toStartOfFile(STATE.file)
            
                if not STATE.contentProblem:
                    STATE.content = ""
                    STATE.content = STATE.file.read().decode('utf-8')
                    if STATE.content == STATE.fileData :
                        st.write(STATE.general_text_dict['correct_file'])
                    else:
                        show_download_button(STATE.newFileName, STATE.fileData)
                else:
                    errDisplay(STATE.errMessageLog)
