"""
Streamlit Application
Marie FU
"""

from datetime import datetime
import json
import streamlit as st
import src.basic as tmp
from src.gerneralFileOP import *

STATE = st.session_state

# Code Start

tmp.base("ElementToTSV")

st.write(st.session_state.general_text_dict["description"])

STATE.newFileName = st.text_input(label=STATE.general_text_dict["file_name_input"],placeholder="File.tsv")

st.file_uploader(st.session_state.general_text_dict['file'], type="json", key="file")

# checking if a file is uploaded
if STATE.file is not None:

    # make sure to be at start of file before any operation
    # file pointer will be reset each time after each file operation
    toStartOfFile(STATE.file)

    # first utilisation of errMessageLog, contain the log of every error message encountered
    STATE.errMessageLog = ""
    
    # check new file name given by user
    STATE.newFileName = checkNewFileName(STATE.newFileName, STATE.file.name)
    if STATE.newFileName is None:
        STATE.errMessageLog += st.session_state.general_text_dict['err_file_name']

    # check file encoding
    content = STATE.file.read()
    STATE.encoding = chardet.detect(content)["encoding"]
    toStartOfFile(STATE.file)
    if STATE.encoding is None:
        STATE.errMessageLog += st.session_state.general_text_dict['err_file_encoding']

    # continue only if file name and file encoding are in a correct
    if STATE.errMessageLog != "":
        errDisplay(STATE.errMessageLog)
    elif STATE.encoding is not None and STATE.newFileName is not None:
        STATE.fileData = "Source\tPublication Year\tPublication Month\tPublication Day\tAuthors\tTitle\tAbstract\n"

        STATE.content = json.loads(STATE.file.read().decode(STATE.encoding))

        source = f"Element chatroom {STATE.content['room_name']}"
        if STATE.content["topic"] != "":
            source += f", {STATE.content['topic']}"

        try:
            for msg in STATE.content["messages"]:
                date = str(datetime.fromtimestamp(msg['origin_server_ts']/1000.0))[:10].replace("-","\t")
                if msg["type"] == "m.room.message":
                    if msg["content"] != {}:
                        if msg["content"]["msgtype"] == "m.text":
                            STATE.fileData += f"{source}\t{date}\t{msg['sender']}\t{msg['event_id']}\t{msg['content']['body']}".replace("\n"," ") + "\n"

            show_download_button(STATE.newFileName, STATE.fileData)
        except:
            st.write("file incorrect")
            print(msg["content"])

    STATE.file.close()
