"""
Streamlit Application
Loïc Chapron
"""

import streamlit as st
import requests as req
import json
import time
from datetime import datetime
from json import JSONDecodeError
import src.basic as tmp


tmp.base("IsidoreToGarganText")

numberReplies = 500  # Dont' exceed 1 000
limitItems = 5000  # Can't be superior of 10 times numberReplies
retryTime = 2

## Connect to Isidore API to get the numbers of docs from the research
def loadApiIsidoreNumberFile(search, language):
    while (True):
        url = 'https://api.isidore.science/resource/search?q=' + search + \
            '&output=json&replies=10&language=http://lexvo.org/id/iso639-3/' + language
        st.session_state.resp = req.get(url)
        print(url)
        if st.session_state.resp.ok:
            try:
                jsontxt = json.loads(st.session_state.resp.content)
                docs = jsontxt["response"]["replies"]["meta"]["@items"]
                break
            except JSONDecodeError:
                continue
        else:
            docs = 0
            break

    return docs

## Connect to Isidore API to get the documents from the pages
def loadApiIsidorePage(search, language, page):
    url = 'https://api.isidore.science/resource/search?q=' + search + '&output=json&replies=' + \
        str(numberReplies) + '&page=' + str(page) + \
        '&language=http://lexvo.org/id/iso639-3/' + language
    resp = req.get(url)
    print(url)
    try:
        jsontxt = json.loads(resp.content)
        docs = jsontxt["response"]["replies"]["content"]["reply"]
    except Exception as e:
        docs = 0

    return docs


def create_output(search, language, nb_doc):
    output = "title\tsource\tpublication_year\tpublication_month\tpublication_day\tabstract\tauthors\n"
    nb = 0
    ## nb is used to return ther number of file with 
    for i in range(1, nb_doc//numberReplies + 1):
        while (True):
            txt = loadApiIsidorePage(search, language, i)
            if txt != 0:
                break
            time.sleep(retryTime)
            print('Retry')
        tmp, nb_tmp = createFile(txt, numberReplies, language)
        output += tmp
        nb += nb_tmp
    
    ## If their is still some document do find (for exampe with 1160 documents, their is still 160 documents to find after the first part)
    if nb_doc % numberReplies != 0:
        while (True):
            txt = loadApiIsidorePage(search, language, nb_doc//numberReplies + 1)
            if txt != 0:
                break
            time.sleep(retryTime)
            print('Retry')
        tmp, nb_tmp = createFile(txt, nb_doc % numberReplies, language)
        output += tmp
        nb += nb_tmp
    return output, nb


def createFile(docs, limit, language):

    # Output text
    output = ''
    i = 0
    nb = 0
    for doc in docs:
        if st.session_state.nb_doc == 1:
            doc = docs
        if (i == limit):
            break
        i += 1

        # Title
        title = doc["isidore"]["title"]
        tmp = doc["@uri"] + " "
        if (type(title) == list):
            for lang in title:
                if type(lang) != str and lang['@xml:lang'] == language[:2]:
                        if "$" in lang.keys():
                            tmp += lang['$']
        title = tmp

        # Source
        source = doc["isidore"]['source']['organization']['name'][3]

        # Author
        if doc['isidore']['enrichedCreators'] != []:
            list_author = doc["isidore"]["enrichedCreators"]["creator"]
            authors = []
            if (type(list_author) == list):
                for author in list_author:
                    authors.append(author["@normalizedAuthor"])
                authors = '; '.join(authors)
            else:
                authors = list_author["@normalizedAuthor"]
        else:
            authors = 'Unknown'

        # Abstract
        if 'abstract' in doc['isidore'].keys() and doc["isidore"]["abstract"] != []:
            abstracts = doc["isidore"]["abstract"]

            if type(abstracts) == list:
                tmp = ''
                for lang in abstracts:
                    if type(lang) == dict and "@xml:lang" in lang.keys():
                        if lang['@xml:lang'] == language[:2] and "$" in lang.keys():
                            tmp = lang['$']
                        
                if tmp == '':
                    if type(abstracts[0]) == str:
                        abstract = abstracts[0]
                    elif "$" in abstracts[0].keys():
                        abstract = abstracts[0]['$']
                else:
                    abstract = tmp

            elif type(abstracts) == dict:
                if "@xml:lang" in abstracts.keys():
                    if abstracts['@xml:lang'] == language[:2] and "$" in abstracts.keys():
                        abstract = abstracts['$']

        else:
            abstract = 'No abstract'
        
        if 'types' in doc['isidore'].keys():
            if type(doc['isidore']['types']['type']) == str and doc['isidore']['types']['type'] in ['Books', 'text']:
                nb += 1
            elif type(doc['isidore']['types']['type']) == dict and doc['isidore']['types']['type']['$'] in ['Books', 'text']:
                nb += 1
        else:
            print(title)

        # Publication Date
        try:
            pdate_ = doc['isidore']["date"]["@endDate"].split('-')
        except KeyError:
            pdate_ = doc['isidore']["date"]["@origin"].split('-')
        pdate = ""
        if pdate_ is not None:
            if len(pdate_) == 1:
                pdate = pdate_[0] + "\t1\t1"
            elif len(pdate_) == 2:
                pdate = "\t".join(pdate_) + "\t1"
            else:
                pdate = "\t".join(pdate_)
        else:
            pdate = "1\t1\t1"

        abstract = abstract.replace('\t', ' ').replace('\n', ' ')
        title = title.replace('\t', ' ').replace('\n', ' ')
        source = source.replace('\t', ' ').replace('\n', ' ')

        # Output
        row = str(title) + "\t" + source + "\t" + str(pdate) + \
            "\t" + abstract + "\t" + authors + "\n"
        output += row

    return output, nb


# Streamlit


lang = {
    'Français': 'fra',
    'Anglais': 'eng',
    'Espagnol': 'spa',
    'Italien': 'ita',
    'Allemand': 'deu',
}

# Can be added but low result
# 'Polonais' : 'nld',
# 'Portugais' : 'por',
# 'Russe' : 'rus'

if 'stage_isidore' not in st.session_state:
    st.session_state.stage_isidore = 0
    st.session_state.nb_wanted = 1


def set_stage(stage):
    st.session_state.stage_isidore = stage
    st.session_state.output = ''


# Image
col1, col2 = st.columns(2)
with col1:
    st.image('img/isidore_logo.png')
with col2:
    st.image('img/gargantext_logo.jpg')

# Form
form = st.form('api')

form.write(st.session_state.general_text_dict['text'])
search = form.text_input(st.session_state.general_text_dict['keyword'])
language = form.selectbox(
    st.session_state.general_text_dict['lang'], lang.keys())

form.form_submit_button(
    st.session_state.general_text_dict['submit'], on_click=set_stage, args=(1,))

# API and Slider
if st.session_state.stage_isidore > 0:

    # Only call first time and after an update in the first form
    if 'search' not in st.session_state or 'language' not in st.session_state or search != st.session_state.search or language != st.session_state.language:
        with st.spinner(st.session_state.general_text_dict['load_api']):
            if language is None:
                language = "Français"
            nb_doc = int(loadApiIsidoreNumberFile(search, lang[language]))
        st.session_state.nb_doc = nb_doc
        if nb_doc != 0:
            st.session_state.search = search
            st.session_state.language = language

    if st.session_state.nb_doc != 0:

        # Form with slider
        form2 = st.form('my_form2')
        form2.write(
            st.session_state.general_text_dict['nb_doc'] + str(st.session_state.nb_doc))

        if st.session_state.nb_doc > limitItems:
            form2.write(st.session_state.general_text_dict['perform1'] + str(
                limitItems) + st.session_state.general_text_dict['perform2'])
            st.session_state.nb_wanted = form2.slider(
                st.session_state.general_text_dict['nb_taken'], 10, limitItems, 10, 10)
        else:
            st.session_state.nb_wanted = form2.slider(
                st.session_state.general_text_dict['nb_taken'], 10, int(st.session_state.nb_doc), 10, 10)
        form2.form_submit_button(
            st.session_state.general_text_dict['submit'], on_click=set_stage, args=(2,))

    else:
        st.write(st.session_state.general_text_dict['overload_api'])


# Download
if st.session_state.stage_isidore > 1:

    with st.spinner(st.session_state.general_text_dict['createTSV']):
        if st.session_state.output == '':
            st.session_state.output, st.session_state.nb_bad_file = create_output(
                st.session_state.search, lang[st.session_state.language], st.session_state.nb_wanted)

    st.write(st.session_state.general_text_dict['doc_abstract1'] + str(
        st.session_state.nb_bad_file) + st.session_state.general_text_dict['doc_abstract2'])
    fileName = "isidoreOutput_" + str(datetime.now().strftime("%Y-%m-%d_%H:%M:%S")) + '.csv'
    st.download_button('Download TSV', st.session_state.output, fileName)
